from MoleKing import Molecule

class TestMolecule():

    def test_add(self):
        mol = Molecule()
        mol.addAtom('H', 0, 0, 0)
        mol.addAtom('H', 0, 0, 1)
        for atom in mol:
            assert atom.getAtomicSymbol() == 'H'

    def test_len(self):
        mol = Molecule()
        mol.addAtom('H', 0, 0, 0)
        mol.addAtom('H', 0, 0, 1)
        assert len(mol) == 2 

    def test_getBonds(self):
        mol = Molecule()
        mol.addAtom('O', 0, 0, 0)
        mol.addAtom('H', 0, 0, 1)
        mol.addAtom('H', 0, 1, 0)
        assert [0, 1] in mol.getIRCBonds()

    def test_getangles(self):
        mol = Molecule()
        mol.addAtom('O', 0, 0, 0)
        mol.addAtom('H', 0, 0, 1)
        mol.addAtom('H', 0, 1, 0)
        assert [1, 0, 2] in mol.getIRCAngles()

    def test_getdihedrals(self):
        mol = Molecule()
        mol.addAtom('C', 0, 0, 0)
        mol.addAtom('H', 0, 0, 1)
        mol.addAtom('H', 0, 1, 0)
        mol.addAtom('O', 1, 0, 0)
        mol.addAtom('H', 2, 0, 0)
        assert [1, 0, 3, 4] in mol.getIRCDihedrals()

    def test_center(self):
        mol = Molecule()
        mol.addAtom('O', 0, 0, 0)
        mol.addAtom('H', 0, 0, 1)
        mol.addAtom('H', 0, 1, 0)
        C = mol.getMassCenter()
        assert C.getCoords('c')[1] < 0.06 
    
    def test_translate(self):
        mol = Molecule()
        mol.addAtom('O', 0, 0, 0)
        mol.addAtom('H', 0, 0, 1)
        mol.addAtom('H', 0, 1, 0)
        mol.moveMassCenter(10, 10, 10)
        C = mol.getMassCenter()
        assert C.getCoords('c') == [10.0, 10.0, 10.0]
    
    def test_VDW(self):
        mol = Molecule()
        mol.setVDWRatio(2)
        assert mol.getVDWRatio() == 2



from MoleKing import G16LOGfile, Psi4OUTfile
import os
import platform

class TestG16Output:
    @classmethod
    def setup_class(cls):
        cls.home_path = os.path.abspath(os.path.dirname(__file__))
        print(f"Home path: {cls.home_path}")

    def test_exist(self):
        assert os.path.exists(os.path.join(self.home_path, 'MK_test1.log')), "Log file does not exist"
        assert os.path.exists(os.path.join(self.home_path, 'MK_orbitals.log')), "Orbitals file does not exist"
        assert os.path.exists(os.path.join(self.home_path, 'MK_link.log')), "Link file does not exist"
        assert os.path.exists(os.path.join(self.home_path, 'MK_link2.log')), "Link2 file does not exist"
        assert os.path.exists(os.path.join(self.home_path, 'MK_polar.log')), "Polar file does not exist"

    def test_reader(self):
        log_path = os.path.join(self.home_path, 'MK_test1.log')
        file = G16LOGfile(log_path).__str__()
        if platform.system() == 'Windows':
            assert file == "G16LOGFile: Calculation of {} done in 2024, with the level of theory B3LYP/6-31+G(d) (6D, 7F) and SCF energy of -155.045622 Hartrees.".format(log_path)
        else:
            assert file == "G16LOGFile: Calculation of MK_test1.log done in 2024, with the level of theory B3LYP/6-31+G(d) (6D, 7F) and SCF energy of -155.045622 Hartrees."

    def test_getMol(self):
        Mol = G16LOGfile(os.path.join(self.home_path, 'MK_test1.log')).getMolecule()
        for atom in Mol:
            assert atom.getAtomicSymbol() in ['H', 'C', 'O']

    def test_getDipole(self):
        Ob = G16LOGfile(os.path.join(self.home_path,'MK_test1.log')).getDipole()
        assert Ob == 1.8039

    def test_getOrbitals(self):
        Ob = G16LOGfile(os.path.join(self.home_path,'MK_orbitals.log')).getOrbitals()
        assert len(Ob['Occupied']) == 13
        assert len(Ob['Unoccupied']) == 56

    def test_HOMO_LUMO(self):       
        Ob = G16LOGfile(os.path.join(self.home_path,'MK_orbitals.log'))
        assert Ob.getHOMO() == -0.27814
        assert Ob.getHOMO(-2) == -0.36919

        assert Ob.getLUMO() == 0.00034
        assert Ob.getLUMO(+4) == 0.07421

    def test_DetectLinks(self):
        Ob1 = G16LOGfile(os.path.join(self.home_path,'MK_link.log'), link=1)
        Ob2 = G16LOGfile(os.path.join(self.home_path,'MK_link.log'), link=2)

        assert Ob1.getEnergy() == -155.045622249
        assert Ob2.getEnergy() == -154.96646388

    def test_LinkOrbitals(self):
        Ob1 = G16LOGfile(os.path.join(self.home_path,'MK_link2.log'), link=1).getOrbitals()
        Ob2 = G16LOGfile(os.path.join(self.home_path,'MK_link2.log'), link=2).getOrbitals()      

        assert len(Ob1['Occupied']) == 13
        assert len(Ob2['Occupied']) == 5

    def test_getAlpha(self):
        f = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getFrequency()
        a1 = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getAlpha(unit='esu', frequency=f[0])['iso']
        a2 = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getAlpha(unit='SI', frequency=f[1])['xx']

        assert f == [0.0, 656.3, 587.6, 486.1]
        assert a1 == 95.8572
        assert a2 == 103.56

    def test_getBeta(self):
        f = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getFrequency()
        b1 = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getBeta(unit='esu', frequency=f[0])['||']
        b2 = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getBeta(unit='au', frequency=f[1], BSHG=1)['_|_(z)']

        assert b1 == 0.3999
        assert b2 == -0.0219886

    def test_getGamma(self):
        f = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getFrequency()
        g1 = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getGamma(unit='esu', frequency=f[0])['||']
        g2 = G16LOGfile(os.path.join(self.home_path,'MK_polar.log'), polarAsw=True).getGamma(unit='au', frequency=f[1], GSHG=1)['xxxx']

        assert g1 == 31.0094
        assert g2 == 61491.6
class TestPSI4Output():

    @classmethod
    def setup_class(cls):
        cls.home_path = os.path.abspath(os.path.dirname(__file__))

    def test_psi4(self): 
        file = Psi4OUTfile(os.path.join(self.home_path,'MK_Test.out'))
        assert file.getMul() == 1
        assert file.getCharge() == 0

    def test_psi4_geo(self):
        file = Psi4OUTfile(os.path.join(self.home_path,'MK_Test.out'))
        assert file.getMolecule().__str__() == 'Molecule BR_{4}O_{2}N_{2}C_{44}H_{30}, with charge 0 and multiplicity 1'
        assert file.getCharge() == 0



#!/usr/bin/env python3

import re

import ghapi.all
from rich import print
from rich.syntax import Syntax

ENTRY = re.compile(
    r"""
    Suggested \s changelog \s entry:
    .*
    ```rst
    \s*
    (.*?)
    \s*
    ```
""",
    re.DOTALL | re.VERBOSE,
)

print()


api = ghapi.all.GhApi(owner="pybind", repo="pybind11")

issues_pages = ghapi.page.paged(
    api.issues.list_for_repo, labels="needs changelog", state="closed"
)
issues = (issue for page in issues_pages for issue in page)
missing = []

for issue in issues:
    changelog = ENTRY.findall(issue.body or "")
    if not changelog or not changelog[0]:
        missing.append(issue)
    else:
        (msg,) = changelog
        if not msg.startswith("* "):
            msg = "* " + msg
        if not msg.endswith("."):
            msg += "."

        msg += f"\n  `#{issue.number} <{issue.html_url}>`_"

        print(Syntax(msg, "rst", theme="ansi_light", word_wrap=True))
        print()

if missing:
    print()
    print("[blue]" + "-" * 30)
    print()

    for issue in missing:
        print(f"[red bold]Missing:[/red bold][red] {issue.title}")
        print(f"[red]  {issue.html_url}\n")

    print("[bold]Template:\n")
    msg = "## Suggested changelog entry:\n\n```rst\n\n```"
    print(Syntax(msg, "md", theme="ansi_light"))

print()



import os
import re
import subprocess
import sys
from pathlib import Path

from setuptools import Extension, setup, find_packages
from setuptools.command.build_ext import build_ext


# Convert distutils Windows platform specifiers to CMake -A arguments
PLAT_TO_CMAKE = {
    "win32": "Win32",
    "win-amd64": "x64",
    "win-arm32": "ARM",
    "win-arm64": "ARM64",
}


# A CMakeExtension needs a sourcedir instead of a file list.
# The name must be the _single_ output extension from the CMake build.
# If you need multiple extensions, see scikit-build.
class CMakeExtension(Extension):
    def __init__(self, name: str, sourcedir: str = "") -> None:
        super().__init__(name, sources=[])
        self.sourcedir = os.fspath(Path(sourcedir).resolve())


class CMakeBuild(build_ext):
    def build_extension(self, ext: CMakeExtension) -> None:
        # Must be in this form due to bug in .resolve() only fixed in Python 3.10+
        ext_fullpath = Path.cwd() / self.get_ext_fullpath(ext.name)  # type: ignore[no-untyped-call]
        extdir = ext_fullpath.parent.resolve()

        # Using this requires trailing slash for auto-detection & inclusion of
        # auxiliary "native" libs

        debug = int(os.environ.get("DEBUG", 0)) if self.debug is None else self.debug
        cfg = "Debug" if debug else "Release"

        # CMake lets you override the generator - we need to check this.
        # Can be set with Conda-Build, for example.
        cmake_generator = os.environ.get("CMAKE_GENERATOR", "")

        # Set Python_EXECUTABLE instead if you use PYBIND11_FINDPYTHON
        # EXAMPLE_VERSION_INFO shows you how to pass a value into the C++ code
        # from Python.
        cmake_args = [
            f"-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={extdir}{os.sep}",
            f"-DPYTHON_EXECUTABLE={sys.executable}",
            f"-DCMAKE_BUILD_TYPE={cfg}",  # not used on MSVC, but no harm
            f"-DBuild_Python=ON",
        ]
        build_args = []
        # Adding CMake arguments set as environment variable
        # (needed e.g. to build for ARM OSx on conda-forge)
        if "CMAKE_ARGS" in os.environ:
            cmake_args += [item for item in os.environ["CMAKE_ARGS"].split(" ") if item]

        # In this example, we pass in the version to C++. You might not need to.
        #cmake_args += [f"-DEXAMPLE_VERSION_INFO={self.distribution.get_version()}"]  # type: ignore[attr-defined]

        if self.compiler.compiler_type != "msvc":
            # Using Ninja-build since it a) is available as a wheel and b)
            # multithreads automatically. MSVC would require all variables be
            # exported for Ninja to pick it up, which is a little tricky to do.
            # Users can override the generator with CMAKE_GENERATOR in CMake
            # 3.15+.
            if not cmake_generator or cmake_generator == "Ninja":
                try:
                    import ninja  # noqa: F401

                    ninja_executable_path = Path(ninja.BIN_DIR) / "ninja"
                    cmake_args += [
                        "-GNinja",
                        f"-DCMAKE_MAKE_PROGRAM:FILEPATH={ninja_executable_path}",
                        f"-DBuild_Python=ON",
                    ]
                except ImportError:
                    pass

        else:

            # Single config generators are handled "normally"
            single_config = any(x in cmake_generator for x in {"NMake", "Ninja"})

            # CMake allows an arch-in-generator style for backward compatibility
            contains_arch = any(x in cmake_generator for x in {"ARM", "Win64"})

            # Specify the arch if using MSVC generator, but only if it doesn't
            # contain a backward-compatibility arch spec already in the
            # generator name.
            if not single_config and not contains_arch:
                cmake_args += ["-A", PLAT_TO_CMAKE[self.plat_name]]

            # Multi-config generators have a different way to specify configs
            if not single_config:
                cmake_args += [
                    f"-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{cfg.upper()}={extdir}"
                ]
                build_args += ["--config", cfg]

        if sys.platform.startswith("darwin"):
            # Cross-compile support for macOS - respect ARCHFLAGS if set
            archs = re.findall(r"-arch (\S+)", os.environ.get("ARCHFLAGS", ""))
            if archs:
                cmake_args += ["-DCMAKE_OSX_ARCHITECTURES={}".format(";".join(archs))]

        # Set CMAKE_BUILD_PARALLEL_LEVEL to control the parallel build level
        # across all generators.
        if "CMAKE_BUILD_PARALLEL_LEVEL" not in os.environ:
            # self.parallel is a Python 3 only way to set parallel jobs by hand
            # using -j in the build_ext call, not supported by pip or PyPA-build.
            if hasattr(self, "parallel") and self.parallel:
                # CMake 3.12+ only.
                build_args += [f"-j{self.parallel}"]

        build_temp = Path(self.build_temp) / ext.name
        if not build_temp.exists():
            build_temp.mkdir(parents=True)

        subprocess.run(
            ["cmake", ext.sourcedir] + cmake_args, cwd=build_temp, check=True
        )
        subprocess.run(
            ["cmake", "--build", "."] + build_args, cwd=build_temp, check=True
        )

# The information here can also be placed in setup.cfg - better separation of
# logic and declaration, and simpler if you include description/version in a file.


def get_version_from_cmakelists(file_path):
    arq = open(file_path, 'r').readlines()
    for line in arq:
        if "MoleKing VERSION" in line:
            return line.split(' ')[-1].split(')')[0]

setup(
    name="MoleKing",
    version = get_version_from_cmakelists('CMakeLists.txt'),
    author="LEEDMOL Research Group",
    author_email="mateus_barbosa@ufg.br",
    description="MoleKing is a python module for chemists aiming to add common principles to python. This module adds new types of python variables, MoleKing_Molecule; MoleKing_Atom; MoleKing_SupraMolecule, and MoleKing_Output, alongside many features considered common knowledge among chemists.",
    long_description="MoleKing is a Python module written in C++ with pybind11 Linkage under LEEDMOL Research Group. This module contains several useful classes for those who program python scripts aimed at theoretical chemistry. This package's main goal is to introduce chemistry concepts, such as Molecules, Atoms, and Geometries, to python, making programming more intuitive and understandable to chemists. Additionally, MoleKing is capable of reading and writing inputs and outputs files for several theoretical chemistry programs.",
    ext_modules=[CMakeExtension("MoleKing")],
    packages=find_packages(),
    include_package_data=True,
    package_data={"":["tests/*.log", "tests/*.out"]},
    cmdclass={"build_ext": CMakeBuild},
    zip_safe=False,
    extras_require={"test": ["pytest>=6.0"]},
    python_requires=">=3.8",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: C++",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Chemistry",
    ],
    license="LICENSE",
)
